<?php

namespace FluentSupportPro\App\Services\FileUploadIntegration\GoogleDrive;


use FluentSupport\Framework\Support\Arr;

class GoogleDriveApi
{
    private $api = 'https://www.googleapis.com/';

    public function uploadToDrive($file, $ticketId = 0)
    {
        $accessToken = GoogleDriveHelper::getAccessToken();

        if (is_wp_error($accessToken)) {
            return $accessToken;
        }

        $fileName = 'ticket_' . $ticketId . '__' . $file->id . '__' . $file->title;
        $mimeType = $file->file_type;
        $fileContent = file_get_contents($file->file_path);

        $url = $this->api . 'upload/drive/v3/files?uploadType=multipart';

        $boundary = wp_generate_password(24);
        $headers = array(
            'Authorization' => 'Bearer ' . $accessToken,
            'Content-Type'  => "multipart/related; boundary={$boundary}",
        );

        $folderId = $this->getParentFolderId($ticketId);

        $body = "--$boundary\r\n";
        $body .= "Content-Type: application/json; charset=UTF-8\r\n\r\n";
        $body .= json_encode(array(
                'name'    => $fileName,
                'parents' => array($folderId)
            )) . "\r\n";

        $body .= "--$boundary\r\n" .
            "Content-Type: $mimeType\r\n" .
            "Content-Transfer-Encoding: base64\r\n\r\n" .
            base64_encode($fileContent) .
            "\r\n--$boundary--\r\n";

        $headers['Content-Length'] = strlen($body);

        $args = array(
            'headers' => $headers,
            'body'    => $body,
            'method'  => 'POST',
            'timeout' => 30
        );

        $response = wp_remote_post($url, $args);
        $responseBody = $this->verifyAndFetchResponse($response, 'id');

        if (is_wp_error($responseBody)) {
            return $responseBody;
        }

        $link = $this->genarateShareLink($responseBody['id']);

        if (is_wp_error($link)) {
            return $link;
        }

        return [
            'full_url' => $link,
            'file_id'  => $responseBody['id'],
        ];
    }

    // Generate share link
    private function genarateShareLink($fileId)
    {
        $access_token = GoogleDriveHelper::getAccessToken();

        // Set the permission
        $url = 'https://www.googleapis.com/drive/v3/files/' . $fileId . '/permissions';

        $args = array(
            'method'  => 'POST',
            'headers' => array(
                'Authorization' => 'Bearer ' . $access_token,
                'Content-Type'  => 'application/json'
            ),
            'body'    => json_encode(array(
                'role' => 'reader',
                'type' => 'anyone'
            ))
        );
        $response = wp_remote_post($url, $args);

        $response = $this->verifyAndFetchResponse($response, 'id');
        if (is_wp_error($response)) {
            return $response;
        }

        // If the permission is set successfully, retrieve the shareable link
        $url = 'https://www.googleapis.com/drive/v3/files/' . $fileId . '?fields=webViewLink';
        $headers = array(
            'Authorization' => 'Bearer ' . $access_token
        );
        $args = array(
            'headers' => $headers
        );
        $response = wp_remote_get($url, $args);
        $response = $this->verifyAndFetchResponse($response, 'webViewLink');
        if (is_wp_error($response)) {
            return $response;
        }

        return $response['webViewLink'];
    }

    public function delete($attachment)
    {

    }

    public function verifyAndFetchResponse($response, $verifyKey = '')
    {
        if (is_wp_error($response)) {
            return $response;
        }

        $status_code = wp_remote_retrieve_response_code($response);
        if ($status_code > 299) {
            $error = wp_remote_retrieve_response_message($response);
            $errorText = sprintf(__('Unable to upload file in Google Drive %s', 'fluent-support-pro'), $error);
            return new \WP_Error('gdrive_upload_failed', $errorText);
        }

        $responseBody = wp_remote_retrieve_body($response);
        $responseBody = json_decode($responseBody, true);

        if ($verifyKey) {
            if (!isset($responseBody[$verifyKey])) {
                $errorText = __('Unable to upload file in google drive', 'fluent-support-pro');
                return new \WP_Error('gdrive_upload_failed', $errorText);
            }
        }

        return $responseBody;
    }

    public function getParentFolderId($ticketId = null)
    {
        $googleDrive = GoogleDriveHelper::getSettings();
        $folderId = Arr::get($googleDrive, 'folder_id');

        if ($folderId) {
            return $folderId;
        }

        // Create the folder
        $url = 'https://www.googleapis.com/drive/v3/files';

        $folderName = 'fs-files-for-' . str_replace(['https://', 'http://', '/'], '', site_url());

        $args = array(
            'method'  => 'POST',
            'headers' => [
                'Authorization' => 'Bearer ' . GoogleDriveHelper::getAccessToken(),
                'Content-Type'  => 'application/json'
            ],
            'body'    => json_encode(array(
                'name'     => $folderName,
                'mimeType' => 'application/vnd.google-apps.folder'
            ))
        );

        $response = wp_remote_post($url, $args);
        $response = $this->verifyAndFetchResponse($response, 'id');

        if (is_wp_error($response)) {
            return $response;
        }

        $settings = GoogleDriveHelper::getSettings();

        $settings['folder_id'] = $response['id'];
        $settings['folder_name'] = $response['name'];

        GoogleDriveHelper::updateSettings($settings);

        return $response['id'];
    }

}
